@@ -5,11 +5,13 @@ import (
 	"io/ioutil"
 	"os"
 
+	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/rest"
 	cmdutil "k8s.io/kubectl/pkg/cmd/util"
+	"k8s.io/kubectl/pkg/util/openapi"
 
 	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
 )
@@ -47,21 +49,38 @@ type helper struct {
 	kubeconfig     []byte
 	restConfig     *rest.Config
 	getFactory     func(namespace string) (cmdutil.Factory, error)
+	openAPISchema  openapi.Resources
+}
+
+// cacheOpenAPISchema builds the very expensive OpenAPISchema (>3s commonly) once, and stores
+// the resulting schema on the helper for re-use, particularly in Apply when run many times against
+// one cluster.
+func (r *helper) cacheOpenAPISchema() error {
+	f, err := r.getFactory("")
+	if err != nil {
+		return errors.Wrap(err, "could not get factory")
+	}
+	r.openAPISchema, err = f.OpenAPISchema()
+	if err != nil {
+		return errors.Wrap(err, "error getting OpenAPISchema")
+	}
+	return nil
 }
 
 // NewHelperFromRESTConfig returns a new object that allows apply and patch operations
-func NewHelperFromRESTConfig(restConfig *rest.Config, logger log.FieldLogger) Helper {
+func NewHelperFromRESTConfig(restConfig *rest.Config, logger log.FieldLogger) (Helper, error) {
 	r := &helper{
 		logger:     logger,
 		cacheDir:   getCacheDir(logger),
 		restConfig: restConfig,
 	}
 	r.getFactory = r.getRESTConfigFactory
-	return r
+	err := r.cacheOpenAPISchema()
+	return r, err
 }
 
 // NewHelperWithMetricsFromRESTConfig returns a new object that allows apply and patch operations, with metrics tracking enabled.
-func NewHelperWithMetricsFromRESTConfig(restConfig *rest.Config, controllerName hivev1.ControllerName, logger log.FieldLogger) Helper {
+func NewHelperWithMetricsFromRESTConfig(restConfig *rest.Config, controllerName hivev1.ControllerName, logger log.FieldLogger) (Helper, error) {
 	r := &helper{
 		logger:         logger,
 		metricsEnabled: true,
@@ -70,33 +89,20 @@ func NewHelperWithMetricsFromRESTConfig(restConfig *rest.Config, controllerName
 		restConfig:     restConfig,
 	}
 	r.getFactory = r.getRESTConfigFactory
-	return r
-}
-
-// NewHelperWithMetrics returns a new object that allows apply and patch operations, with metrics tracking enabled.
-
-func NewHelperWithMetrics(kubeconfig []byte, controllerName hivev1.ControllerName, remote bool, logger log.FieldLogger) Helper {
-	r := &helper{
-		logger:         logger,
-		controllerName: controllerName,
-		metricsEnabled: true,
-		cacheDir:       getCacheDir(logger),
-		kubeconfig:     kubeconfig,
-		remote:         true,
-	}
-	r.getFactory = r.getKubeconfigFactory
-	return r
+	err := r.cacheOpenAPISchema()
+	return r, err
 }
 
 // NewHelper returns a new object that allows apply and patch operations
-func NewHelper(kubeconfig []byte, logger log.FieldLogger) Helper {
+func NewHelper(kubeconfig []byte, logger log.FieldLogger) (Helper, error) {
 	r := &helper{
 		logger:     logger,
 		cacheDir:   getCacheDir(logger),
 		kubeconfig: kubeconfig,
 	}
 	r.getFactory = r.getKubeconfigFactory
-	return r
+	err := r.cacheOpenAPISchema()
+	return r, err
 }
 
 func getCacheDir(logger log.FieldLogger) string {
